<?php
namespace Financeiro\Logic\ContasPagar\Cnab\Retorno;

use Geral\Entities\Empresa;
use Geral\Util\Ambiente;
use \Financeiro\Logic\ContasPagar\Cnab\Retorno\Retorno;

/**
 * Created by JetBrains PhpStorm.
 * User: alanoliveira
 * Date: 28/04/14
 * Time: 14:15
 * To change this template use File | Settings | File Templates.
 */
class LeitorCnabSantander extends LeitorCnab {

    const CODIGO_COMPENSACAO_SANTANDER = "033";
    CONST ID_TP_ARQUIVO_HEADER_ARQUIVO = '0';
    CONST ID_TP_ARQUIVO_HEADER_LOTE = '1';
    CONST ID_TP_ARQUIVO_SEGMENTO = '3';
    CONST ID_TP_ARQUIVO_TRAILER_LOTE = '5';
    CONST ID_TP_TRAILER_ARQUIVO = '9';
    CONST TIPO_INS_JUR_CPF = 1;
    CONST TIPO_INS_JUR_CNPJ = 2;
    CONST RETORNO = 2;
    CONST SEGMENTO_A = 'A';
    CONST SEGMENTO_J = 'J';
    CONST SEGMENTO_O = 'O';
    CONST SEGMENTO_N = 'N';


    private $cdagencia;
    private $cdconta;
    private $ocorrenciaHeader;
    private $contHeaderLote = 0;


    /**
     * @param $arquivo
     * @param $cdempresa
     * @param $cdbanco
     * @throws \Exception
     */
    public function __construct($arquivo, $cdempresa, $cdbanco){
        try {

            $this->cdbanco = $cdbanco;
            $this->arquivo = $arquivo;
            $this->cdempresa = $cdempresa;

        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function leArquivo(){

        try {
            foreach ($this->arquivo as $linha) {
                $this->contlinha++;

                $this->retorno = new Retorno();

                // Busca o Tipo de Registro do Arquivo
                $idRegistro = $linha[7];

                switch ($idRegistro) {

                    case(self::ID_TP_ARQUIVO_HEADER_ARQUIVO):
                        self::leHeaderArquivo($linha);
                        break;

                    case(self::ID_TP_ARQUIVO_HEADER_LOTE):
                        self::leHeaderLote($linha);
                        break;

                    case(self::ID_TP_ARQUIVO_SEGMENTO):
                        self::leDetalhe($linha);
                        break;

                    case(self::ID_TP_ARQUIVO_TRAILER_LOTE):
                        self::leTrailerLote($linha);
                        break;

                    case(self::ID_TP_TRAILER_ARQUIVO):
                    break;
                }
                $this->retornos[] = $this->retorno;
            }
            self::preparaOcorrencias($this->cdbanco);

        } catch (\Exception $e) {
            throw $e;
        }
    }


    /**
     * @param $cdempresa
     * @throws \Exception
     * @return
     */
    private function getIncricaoEmpresa($cdempresa){
        $empresa = Empresa::findOneByCdempresaAndNrorg($cdempresa, Ambiente::getNrorg());
        if($empresa)
           return $empresa->getNrinsjurempr();
        else
            throw new \Exception ('Empresa não Encontrada.');
    }


    /**
     * // =====================================================================  //
       // ====================  REGISTRO HEADER  DE ARQUIVO  ====================  //
       // =====================================================================  //
     * @param TXT $header
     */
    private function leHeaderArquivo($header){

        // CAMPO: Código do BCO na compensação
        // POSIÇÃO: 1 a 3
        // PICTURE: 9(3)
        if(substr($header,0,3) != self::CODIGO_COMPENSACAO_SANTANDER)
            self::gravaErros($this->contlinha,'Banco não Cadastrado.', true);

        // CAMPO: Tipo de inscrição da empresa
        // POSIÇÃO: 18 a 18
        // PICTURE: 9(1)
        $tipoInscricao = substr($header,17,1);
        if(empty($tipoInscricao))
            self::gravaErros($this->contlinha,'Tipo de Inscrição da Empresa não Informado.', true);

        $nrinsjurempr = self::getIncricaoEmpresa($this->cdempresa);
        // CAMPO: Número de inscrição da empresa
        // POSIÇÃO: 19 a 32
        // PICTURE: 9(14)
        if($tipoInscricao == self::TIPO_INS_JUR_CPF){
            $inscricao = substr($header,21,11);
        }else{
            $inscricao = substr($header,18,14);
        }
        if($nrinsjurempr <> $inscricao)
            self::gravaErros($this->contlinha,'Inscrição da empresa não cadastrada.', true);

        // CAMPO: Código de Remessa/Retorno
        // POSIÇÃO: 143 a 143
        // PICTURE: 9(1)
        if(substr($header,142,1) != self::RETORNO)
            self::gravaErros($this->contlinha,'Código de Retorno Inválido.', true);
    }

    /**
     * // =====================================================================  //
       // ====================  REGISTRO HEADER  DE LOTE  ====================  //
       // =====================================================================  //
     * @param TXT $header
     */
    private function leHeaderLote($header){
        // CAMPO: Número da agência centralizadora
        // POSIÇÃO: 53 a 57
        // PICTURE: 9(5)
        $this->cdagencia = substr($header,52,5);

        // CAMPO: Conta debitada
        // POSIÇÃO: 59 a 70
        // PICTURE: 9(12)
        $this->cdconta = substr($header,58,12);

        // CAMPO: Ocorrências
        // POSIÇÃO: 231 a 240
        // PICTURE: X(10)
        $this->ocorrenciaHeader = substr($header,230,10);
        $this->contHeaderLote++;

    }

    /**
    // =====================================================================  //
    // ====================      REGISTRO DETALHE       ====================  //
    // =====================================================================  //
     * @param $detalhe
     */
    private function leDetalhe($detalhe){

        $segmento = substr($detalhe,13,1);

        if($segmento == self::SEGMENTO_A or $segmento == self::SEGMENTO_J )
        {
            $this->retorno->setIdheaderlote($this->contHeaderLote);

            if($segmento == self::SEGMENTO_A){
                //Registro do segmento A
                $cdfilial = substr($detalhe,177,4);
                $nrlanbolpag = substr($detalhe,181,10);
                $vencimento = (substr($detalhe,93,8));
            }
            else{
                //Registro do segmento j
                $cdfilial = substr($detalhe,182,4);
                $nrlanbolpag = substr($detalhe,186,10);
                $vencimento = (substr($detalhe,91,8));

            }
            $this->retorno->setCdlinha(str_pad($this->contlinha,10,'0',STR_PAD_LEFT));
            $this->retorno->setCdfilial($cdfilial);
            $this->retorno->setNrlanbolpag($nrlanbolpag);
            $this->retorno->setBoleto('BOLETO');
            $this->retorno->setCdconta($this->cdconta);
            $this->retorno->setCdagencia($this->cdagencia);
            $this->retorno->setDtvencimento($vencimento);
            $this->retorno->setCdtpbaixapag(self::getTipoBaixaPagamento($cdfilial, $nrlanbolpag, Ambiente::getNrorg()));
            $this->retorno->setCdreducont(self::getReducont($cdfilial, $nrlanbolpag, Ambiente::getNrorg()));

            // CAMPO: Tipo de movimento   [0 - Inclusão  9 - Exclusão]
            // POSIÇÃO: 15 a 15
            // PICTURE: 9(1)
            $tipomovimento = substr($detalhe,14,1);
            self::tipoMovimento($tipomovimento);

            // CAMPO: Ocorrência
            // POSIÇÃO: 231 a 240
            // PICTURE: X(10)
            $this->retorno->setOcorrencia(substr($detalhe,230,10));


            // =================  REGISTRO DO SEGMENTO J  =================  //
            if($segmento == self::SEGMENTO_J){

                // CAMPO: Nome do Cedente
                // POSIÇÃO: 062 a 091
                // PICTURE: X(30)
                $this->retorno->setNmrazsocforn(substr($detalhe,61,30));

                // CAMPO: Valor do deconto + Abatimento
                // POSIÇÃO: 115 a 129
                // PICTURE: 9(13)V9(2)
                $valordesconto = floatval(substr($detalhe,114,15));
                $this->retorno->setVrdesconto(round($valordesconto/100,2));

                // CAMPO: Valor da mora + Multa
                // POSIÇÃO: 130 a 144
                // PICTURE: 9(13)V9(2)
                $valoracrescimo = floatval(substr($detalhe,129,15));
                $this->retorno->setVracrescimo(round($valoracrescimo/100,2));

                // CAMPO: Data do pagamento
                // POSIÇÃO: 145 a 152
                // PICTURE: 9(8)
                $this->retorno->setDtpagamento(substr($detalhe,144,8));

                // CAMPO: Valor do pagamento
                // POSIÇÃO: 153 a 167
                // PICTURE: 9(13)V9(2)
                $varpago = floatval(substr($detalhe,152,15));
                $this->retorno->setVrpago(round($varpago/100,2));

                // CAMPO: Valor do pagamento
                // POSIÇÃO: 100 a 114
                // PICTURE: 9(13)V9(2)
                $varpago = floatval(substr($detalhe,99,15));
                $this->retorno->setVrfinalpago(round($varpago/100,2));

                // CAMPO: Cód. Refer. atribuído pelo sacado
                // POSIÇÃO: 183 a 202
                // PICTURE: X(20)
                $nrret = substr($detalhe,182,38);
                if(empty($nrret))
                    self::gravaErros($this->contlinha,'Número do Retorno não informado.');
            }else{

                if($segmento == self::SEGMENTO_A){

                    // CAMPO: Nome do Favorecido
                    // POSIÇÃO: 044 a 073
                    // PICTURE: X(30)
                    $this->retorno->setNmrazsocforn(substr($detalhe,43,30));


                    // CAMPO: Data da efetivação do lançamento
                    // POSIÇÃO: 155 a 162
                    // PICTURE: 9(8)
                    $this->retorno->setDtpagamento(substr($detalhe,154,8));

                    // CAMPO: Valor do lançamento
                    // POSIÇÃO: 120 a 134
                    // PICTURE: 9(13)V9(2)
                    $varpago = floatval(substr($detalhe,119,15));
                    $this->retorno->setVrpago(round($varpago/100,2));

                    // CAMPO: Valor Real do lançamento
                    // POSIÇÃO: 163 a 177
                    // PICTURE: 9(13)V9(2)
                    $varpago = floatval(substr($detalhe,162,15));
                    $this->retorno->setVrfinalpago(round($varpago/100,2));

                    // CAMPO: Campo de uso exclusivo da empresa
                    // POSIÇÃO: 180 a 217
                    // PICTURE: X(20)
                    $nrret = substr($detalhe,179,38);
                    if($nrret == false)
                        self::gravaErros($this->contlinha,'Número do Retorno não informado.');
                }

            }

        }

    }

    /**
     * @param $trailerlote
     */
    public function leTrailerLote($trailerlote){

        // CAMPO: Ocorrências
        // POSIÇÃO: 231 a 240
        // PICTURE: X(10)
        $ocorrenciastrailer = trim(substr($trailerlote,230,10));

        /** @var $retorno Retorno  */
        foreach($this->retornos as &$retorno){

            if($retorno->getIdheaderlote() == $this->contHeaderLote){

                //Insere Situação do Cnab
                $retorno->setIdsitucnabp(self::situacaoCnab($retorno->getOcorrencia().
                                                            $ocorrenciastrailer.
                                                            $this->ocorrenciaHeader));

                //Adiciona as ocorrencias do trailler e do header.
                $retorno->setOcorrencia($retorno->getOcorrencia().
                                        $ocorrenciastrailer.
                                        $this->ocorrenciaHeader);
            }
        }
    }

    /**
     * @param $cdocorrencia
     * @return string
     */
    private function situacaoCnab($cdocorrencia){


        if ($cdocorrencia){
            if (strpos($cdocorrencia,'BD') !== false)
               return '20'; // Retorno Ok (Agendado)
            if (strpos($cdocorrencia,'02') !== false)
                return '00'; // Não Agendado
            if (strpos($cdocorrencia,'00') !== false)
                return '90'; // Baixado via CNAB
            if ((strpos($cdocorrencia,'BD') === false) and
               (strpos($cdocorrencia,'00') === false) and
               (strpos($cdocorrencia,'02') === false))
               return '01'; // Retorno com erro
        }
        return '10'; //Sem retorno
    }

}